#version 420

in vec2 uv;
in vec4 smCoords[4];

layout(binding = 0) uniform sampler2D texture;
layout(binding = 1) uniform sampler2D shadowMap1;
layout(binding = 2) uniform sampler2D shadowMap2;
layout(binding = 3) uniform sampler2D shadowMap3;
layout(binding = 4) uniform sampler2D shadowMap4;

out vec3 color;

float calcShadowFactor(vec4 coords, sampler2D shadowMap, float min)
{
	vec2 sm_uvs = coords.xy / coords.w;
	float depth_p = coords.z / coords.w;
	float depth_c = texture2D(shadowMap, sm_uvs).r;
	float visibility = (depth_c < depth_p) ? min : 1.0;

	return visibility;
}

void main()
{
	// Light emission properties
	vec3 lightColor = vec3(1,1,1);
	
	// Material properties
	vec3 materialDiffuseColor = texture2D(texture, uv).rgb;

	// Calculate shadow
	float shadowFactor1 = calcShadowFactor(smCoords[0], shadowMap1, 0.5);
	float shadowFactor2 = calcShadowFactor(smCoords[1], shadowMap2, 0.0);
	float shadowFactor3 = calcShadowFactor(smCoords[2], shadowMap3, 0.0);
	float shadowFactor4 = calcShadowFactor(smCoords[3], shadowMap4, 0.0);

	float shadowFactor = 
		shadowFactor1 + shadowFactor2 + shadowFactor3 + shadowFactor4;
	if (shadowFactor > 1.0)
		shadowFactor = 1.0;

	/*if (shadowFactor1 == 0 && shadowFactor2 == 0 &&
		shadowFactor3 == 0 && shadowFactor4 == 0)
		shadowFactor = shadowFactor1;
	else
		shadowFactor = shadowFactor1;*/

	color = vec3(shadowFactor) * materialDiffuseColor * lightColor;
	//color = vec3(1.0 - (1.0 - depth_p) * 25.0);
	//color = vec3(depth_p);
}